﻿using System;
using System.IO;

namespace Black.NBT
{
    /// <summary>
    /// Représente un TAG_String
    /// </summary>
    public class TagString : Tag, ICloneable
    {
        #region Propriétés

        /// <summary>
        /// Obtient ou définit sa valeur
        /// </summary>
        public string Value
        {
            get
            {
                return (this._value);
            }
            set
            {
                if (this._value != value)
                {
                    this._value = value;
                    this.OnPropertyChanged();
                }
            }
        }

        #endregion

        #region Variables d'instances

        private string _value;

        #endregion

        #region Constructeur de la classe

        /// <summary>
        /// Constructeur de la classe
        /// </summary>
        /// <param name="name">Nom du tag</param>
        public TagString(string name)
            : base(name)
        {
            this.TagType = NBT.TagType.String;
        }

        #endregion

        #region Fonctions internes

        /// <summary>
        /// Ecrit l'instance sur le stream
        /// </summary>
        /// <param name="stream">Instance du flux</param>
        /// <param name="fromList">Si l'écriture vient d'une liste</param>
        internal override void WriteToStream(MemoryStream stream, bool fromList)
        {
            // Si pas d'une liste on inscrit tout 

            if (!fromList)
            {
                // Ecriture du type

                stream.WriteByte((byte)this.TagType);

                // Ecriture du nom


                NBTFile.WriteString(stream, this.Name);
            }

            // Ecriture de la valeur

            NBTFile.WriteString(stream, this.Value);
        }

        #endregion

        #region Fonctions statiques

        /// <summary>
        /// Lit le tag dans le flux
        /// </summary>
        /// <param name="stream">Instance du flux placé</param>
        /// <returns>Instance du Tag</returns>
        public static TagString Read(MemoryStream stream)
        {
            return (TagString.Read(stream, true));
        }

        /// <summary>
        /// Lit le tag dans le flux
        /// </summary>
        /// <param name="stream">Instance du flux placé</param>
        /// <param name="withName">TRUE récupère le nom, FALSE sinon</param>
        /// <returns>Instance du Tag</returns>
        public static TagString Read(MemoryStream stream, bool withName)
        {
            // On récupère son nom

            string name = string.Empty;

            if (withName)
                name = Tag.ReadName(stream);

            // Création de l'instance

            TagString item = new TagString(name);
            item.Value = NBTFile.ReadString(stream);

            // On retourne la valeur

            return (item);
        }

        #endregion

        #region Clonage

        /// <summary>
        /// Clone l'instance
        /// </summary>
        /// <returns>Nouvelle instance</returns>
        object ICloneable.Clone()
        {
            return (this.Clone());
        }

        /// <summary>
        /// Clone l'instance
        /// </summary>
        /// <returns>Nouvelle instance</returns>
        public virtual TagString Clone()
        {
            TagString clone = this.MemberwiseClone() as TagString;

            return (clone);
        }

        #endregion
    }
}
